#!/usr/bin/env python
from distutils.core import setup

setup(name='rigidbody',
      description='Rigid body transforms in 3D',
      version='0.11',
      author='Alex Flint',
      author_email='alex.flint@gmail.com',
      url='https://github.com/alexflint/rigidbody',
      packages=['rigidbody'],
      package_dir={'rigidbody': '.'},
      )
